import QtQuick 2.9
import QtQuick.Controls 2.2
import QtQuick.Controls.Styles 1.4
import QtQuick.Extras 1.4
import QtQuick.Extras.Private 1.0
import QtQuick.Layouts 1.3
import QtQuick.Controls.Material 2.2
import QtGraphicalEffects 1.0

Item {
    id: speedometer
    width: 300
    height: 300

    property alias value: gauge.value
    property alias maximumValue: gauge.maximumValue
    property alias minimumValue: gauge.minimumValue

    property string gaugeText: "km/h"
    property int size: (width < height) ? width : height
    property int borderSize: 0

    property string fontName: "Sans Serif"
    property alias gradient: rect.gradient
    property color backgroundColor: "#000000"
    property color textColor: "#F5F5F5"
    property color foregroundColor: "#66bb6a"
    property color foregroundColorLight: "#00c853"

    // ARREGLAR ESTOS
    property double tickmarkStepSize1: 10
    property int minAngleValue: -145
    property int maxAngleValue: 145

    Item {
        id: container
        anchors.centerIn: parent
        width:  rect.width  + 10+(2 * shadow.radius)
        height: rect.height + 10+(2 * shadow.radius)
        visible: false

        Rectangle {
            id: rect
            anchors.centerIn: parent
            width: size
            height: size
            radius: width*0.5
            color: backgroundColor
            Material.elevation: 20
            border.width: (borderSize === 0) ? 5 : borderSize
            border.color: (borderSize === 0) ? "#00000000" : backgroundColor

            CircularGauge {
                id: gauge
                anchors.centerIn: parent
                width: (borderSize === 0) ? size : size - borderSize
                height: (borderSize === 0) ? size : size - borderSize
                //width: size
                //height: size
                maximumValue: speedometer.maximumValue
                minimumValue: speedometer.minimumValue
                Behavior on value {
                    NumberAnimation {
                        duration: 1000
                    }
                }

                ColumnLayout {
                    anchors.centerIn: parent
                    Text {
                        Layout.alignment: Qt.AlignCenter
                        color: textColor
                        font.pointSize: (0.12*size)
                        text: (gauge.value).toFixed(1)
                        font.family: fontName
                    }
                    Text {
                        Layout.alignment: Qt.AlignCenter
                        color: textColor
                        font.pointSize: (0.05*size)
                        text: gaugeText
                        font.family: fontName
                    }
                }

                style: CircularGaugeStyle {
                    id: style

                    maximumValueAngle: maxAngleValue
                    minimumValueAngle: minAngleValue

                    function degreesToRadians(degrees) {
                        return degrees * (Math.PI / 180);
                    }

                    tickmarkLabel:  Text {
                        text: styleData.value
                        font.pointSize: (0.03 * size)
                        color: textColor
                        antialiasing: true
                        font.family: fontName
                    }

                    tickmarkStepSize: tickmarkStepSize1

                    tickmark: Rectangle {
                        visible: false
                    }

                    minorTickmark: Rectangle {
                        visible: false
                    }

                    needle: Rectangle {
                        //y: outerRadius * (-0.95)
                        //implicitWidth: 10
                        //implicitHeight: 10
                        y: -1 * (outerRadius - implicitWidth/2 /*- outerRadius * 0.01/2*/ - outerRadius * 0.025/2)
                        implicitWidth: outerRadius * 0.065
                        implicitHeight: outerRadius * 0.065
                        radius: width*0.5
                        antialiasing: true
                        color: foregroundColorLight
                    }

                    background: Canvas {

                        onPaint: {
                            var ctx = getContext("2d");
                            ctx.reset();

                            ctx.beginPath();
                            ctx.strokeStyle = foregroundColor;
                            ctx.lineWidth = outerRadius * 0.01;

                            ctx.arc(outerRadius, outerRadius, outerRadius - ctx.lineWidth / 2,
                                degreesToRadians(valueToAngle(0) - 90), degreesToRadians(valueToAngle(gauge.maximumValue) - 90));
                            ctx.stroke();
                        }

                    }

                    foreground: Canvas {
                        id: canvas

                        onPaint: {
                            var ctx = getContext("2d");
                            ctx.reset();

                            ctx.beginPath();
                            ctx.strokeStyle = foregroundColorLight;
                            ctx.lineWidth = outerRadius * 0.025;

                            ctx.arc(outerRadius, outerRadius, outerRadius - ctx.lineWidth / 2,
                                degreesToRadians(valueToAngle(0) - 90), degreesToRadians(valueToAngle(gauge.value) - 90));
                            ctx.stroke();
                        }

                        Connections {
                            target: gauge
                            onValueChanged: canvas.requestPaint();
                        }

                    }
                }
            }
        }
    }

    DropShadow {
        id: shadow
        anchors.fill: source
        cached: true
        horizontalOffset: 0
        verticalOffset: 0
        radius: 10.0
        samples: 16
        color: "#80000000"
        smooth: true
        source: container
    }
}
