#include "backend.h"
#include <stdlib.h>

BackEnd::BackEnd(QObject *parent) : CanBus (parent)
{
    _voltage = 0.0;
    _velocity = 0;
    _rpm = 0;
    _current = 0;
    _height = 0;
    _temp = 0;
    _humidity = 0;
    _pressure = 0;
    _battery = 0;
    if(_canBusDevice!=nullptr){
        QObject::connect(_canBusDevice, &QCanBusDevice::framesReceived, this,&BackEnd::framesReceived);
        QObject::connect(this,&BackEnd::framesReceived,this,&BackEnd::readData);
    }else if (QCanBus::instance()->plugins().contains(QStringLiteral("socketcan"))) {
            _canBusDevice = QCanBus::instance()->createDevice(QStringLiteral("socketcan"), QStringLiteral("vcan1"));
            qDebug() << "Backend ok";
            QObject::connect(_canBusDevice, &QCanBusDevice::framesReceived, this,&BackEnd::framesReceived);
            QObject::connect(this,&BackEnd::framesReceived,this,&BackEnd::readData);
    }else{
        qDebug() <<"The world is burning";
    }


    //qDebug() << "Backend init";
 }


float BackEnd::voltage()
{
    return _voltage;
}

float BackEnd::current(){
    return _current;

}
float BackEnd::battery(){
    return _battery;
}

float BackEnd::temp(){
    return _temp;
}

float BackEnd::humidity(){
    return _humidity;

}

float BackEnd::height(){
    return _height;
}

float BackEnd::pressure()
{
    return _pressure;
}

float BackEnd::velocity(){
    return _velocity;
}

float BackEnd::rpm(){
    return _rpm;
}

void BackEnd::setVoltage(float voltage)
{
    if (_voltage != voltage) {
        _voltage = voltage;
        emit voltageChanged();
    }
}

void BackEnd::setCurrent(float current){
    if(_current!= current){
        _current = current;
        emit currentChanged();
    }
}

void BackEnd::setBattery(float battery){
    if(_battery!=battery){
        _battery = battery;
        emit batteryChanged();
    }
}
void BackEnd::setTemp(float temp){
    if(_temp!=temp){
        _temp = temp;
        emit tempChanged();
    }
}

void BackEnd::setHumidity(float humidity){
    if(_humidity!=humidity){
        _humidity = humidity;
        emit humidityChanged();
    }
}

void BackEnd::setHeight(float height){
    if(_height!=height){
        _height = height;
        emit heightChanged();
    }
}

void BackEnd::setPressure(float pressure){
    if(_pressure!=pressure){
        _pressure=pressure;
        emit pressureChanged();
    }
}
void BackEnd::setVelocity(float velocity){
    if(_velocity!=velocity){
        _velocity = velocity;
        emit velocityChanged();
    }
}

void BackEnd::setRpm(float rpm){
    if(_rpm!=rpm){
        _rpm = rpm;
        emit rpmChanged();
    }
}

void BackEnd::readData(){


    while(_canBusDevice->framesAvailable()>0){
        CanBus::readFrame();
        //qDebug()<<"Frame received";
        if(_frameId == 1){
           // float* f = (float*)_frameData.data();
            //setVoltage(*f);//(_frameData.toFloat());
            setVoltage(_frameData.toFloat());
            if(voltage()<1.5){
                emit voltageWrong();
            }
        } else if(_frameId == 2){
            setVelocity(_frameData.toFloat());
            if(velocity()>70.0){
                emit velocityMax();
            }
        } else if(_frameId == 3){
            setRpm(_frameData.toFloat());
            if(rpm()>3.0){
                emit rpmCritical();
            }

        } else if(_frameId == 4){
            setBattery(_frameData.toFloat());
            if(battery() <10.0){
                emit criticalBattery();
            }
        } else if(_frameId == 5){
            setCurrent(_frameData.toFloat());
            if(current()> 5.0){
                emit currentExceed();
            }
        } else if(_frameId == 6){
            setHeight(_frameData.toFloat());
            if(height()>1000.0){
                emit heightExceed();
            }
        } else if(_frameId == 7){
            setPressure(_frameData.toFloat());
            if(pressure()>1500.0){
                emit pressureExceed();
            }
        } else if(_frameId == 8){
            setTemp(_frameData.toFloat());
            if(temp()>450.0){
                emit tempExceed();
            }
        } else if(_frameId == 9){
            setHumidity(_frameData.toFloat());
            if(humidity()>2.0){
                emit humidityExceed();
            }
        } else{
            qDebug()<< "Data error";
        }

    }

}

QString BackEnd::printFrame(){
   return _frame.toString();
}

