#include "canbus.h"
#include "backend.h"
class BackEnd;
CanBus::CanBus(QObject *parent) : QObject(parent)
{
    _deviceId = 0;
    _loopBackEnable = false;
    _isConnected = false;
    _bitRate = BitrateNull;
    _frame.setFrameType(QCanBusFrame::ErrorFrame);
    if (QCanBus::instance()->plugins().contains(QStringLiteral("socketcan"))) {
        _canBusDevice = QCanBus::instance()->createDevice(QStringLiteral("socketcan"), QStringLiteral("vcan1"));
        qDebug() << "SocketCAN ok";
        _canBusDevice->setConfigurationParameter(QCanBusDevice::CanFdKey,true);
        setBitRate(Bitrate1000000);
    } else {
        qDebug() << "ERROR: CAN BUS DEVICE NOT FOUND";
    }
    QObject::connect(_canBusDevice, &QCanBusDevice::framesReceived, this,&CanBus::framesReceivedLinker);
}

bool CanBus::isConnected()
{
    return _isConnected;
}


bool CanBus::setIsConnected(bool isConnected)
{
    if (!_isConnected && isConnected) {
        if (_canBusDevice->connectDevice()) {
            _isConnected = true;
            emit connected();
            return true;
        }
    } else if (_isConnected && !isConnected) {
        _canBusDevice->disconnectDevice();
        _isConnected = false;
        emit disconnected();
        return true;
    }
    return false;
}

CanBus::BitRate CanBus::bitRate()
{
    return _bitRate;
}

void CanBus::setBitRate(CanBus::BitRate bitRate)
{

    if (_bitRate != bitRate) {
        _canBusDevice->setConfigurationParameter(QCanBusDevice::BitRateKey, bitRate);
         //QVariant a =_canBusDevice->configurationParameter(QCanBusDevice::BitRateKey);
        _bitRate = bitRate;
         emit bitRateChanged();
    }
}

quint32 CanBus::deviceId()
{
    return _deviceId;
}

void CanBus::setDeviceId(quint32 deviceId)
{
    if (_deviceId != deviceId) {
        _deviceId = deviceId;
        emit deviceIdChanged();
    }
}

QCanBusFrame CanBus::frame()
{
    return _frame;
}

quint32 CanBus::frameId(){
    return _frameId;
}

/*QByteArray CanBus::frameData(){
    return _frameData;
}
*/

bool CanBus::sendFrame(quint32 id, const QByteArray &data)
{
    QCanBusFrame frame;
    frame.setFrameId(id);
    frame.setFrameType(QCanBusFrame::DataFrame);
    frame.setPayload(data);
    return _canBusDevice->writeFrame(frame);
}

void CanBus::addMaskFilter(quint32 mask, quint32 filter)
{
    QCanBusDevice::Filter newFilter;
       newFilter.frameId = filter;
       newFilter.frameIdMask = mask;
       _filterList.append(newFilter);
       QVariant data;
       data.setValue<QList<QCanBusDevice::Filter>>(_filterList);
       _canBusDevice->setConfigurationParameter(QCanBusDevice::RawFilterKey, data);
}

void CanBus::loopbackEnable()
{
    if(!_loopBackEnable){
         _canBusDevice->setConfigurationParameter(QCanBusDevice::LoopbackKey,true);
        _loopBackEnable = true;
       // return _loopBackEnable;
    }
}
void CanBus::loopbackDisable(){
    if(_loopBackEnable){
        _loopBackEnable = false;
        _canBusDevice->setConfigurationParameter(QCanBusDevice::LoopbackKey,false);
    }
}
void CanBus::readFrame(){
    _frame = _canBusDevice->readFrame();
    _frameId = _frame.frameId();
    _frameData = _frame.payload();
}
QString CanBus::rawRead()
{
    return _canBusDevice->readFrame().toString();
}

bool CanBus::rawWrite(quint32 id, const QByteArray& data)
{
    return sendFrame(id,data);
}

bool CanBus::connect()
{
    return CanBus::setIsConnected(true);
}

void CanBus::disconnect()
{
    CanBus::setIsConnected(false);
}

quint32 CanBus::framesAvailable()
{
    return _canBusDevice->framesAvailable();
}
void CanBus::framesReceivedLinker(){
    emit framesReceived();
   // qDebug()<<"Frames Received";
}


