#ifndef CANBUS_H
#define CANBUS_H

#include <QDebug>
#include <QObject>
#include <QtSerialBus/QCanBus>
#include <QtSerialBus/QCanBusDevice>
#include <QtSerialBus/QCanBusFrame>

//Q_DECLARE_METATYPE(QCanBusDevice::Filter)
//Q_DECLARE_METATYPE(QList<QCanBusDevice::Filter> *)
class CanBus : public QObject
{
    Q_OBJECT

    Q_PROPERTY(bool isConnected READ isConnected WRITE setIsConnected NOTIFY connected NOTIFY disconnected)
    Q_PROPERTY(BitRate bitRate READ bitRate WRITE setBitRate NOTIFY bitRateChanged)
    Q_PROPERTY(quint32 deviceId READ deviceId WRITE setDeviceId NOTIFY deviceIdChanged)
    Q_PROPERTY(quint32 framesAvailable)
public:
    explicit CanBus(QObject *parent = nullptr);
    Q_ENUMS(QCanBusDevice::CanBusDeviceState)
    Q_ENUMS(QCanBusDevice::CanBusError)

    enum BitRate{
        BitrateNull = 0,
        Bitrate50000 = 50000,
        Bitrate100000 = 100000,
        Bitrate250000 = 250000,
        Bitrate500000 = 500000,
        Bitrate800000 = 800000,
        Bitrate1000000 = 1000000
    };
    Q_ENUMS(BitRate)

    bool isConnected();
    bool setIsConnected(bool isConnected);

    BitRate bitRate();
    Q_INVOKABLE void setBitRate(BitRate bitRate);

    quint32 deviceId();
    void setDeviceId(quint32 deviceId);
    Q_INVOKABLE QCanBusFrame frame();
    Q_INVOKABLE quint32 frameId();
    //Q_INVOKABLE frameData();

    Q_INVOKABLE bool connect();
    Q_INVOKABLE void disconnect();

    Q_INVOKABLE bool sendFrame(quint32 id, const QByteArray &data);

    Q_INVOKABLE void addMaskFilter(quint32 mask, quint32 filter);
    Q_INVOKABLE void loopbackEnable();
    Q_INVOKABLE void loopbackDisable();
    Q_INVOKABLE void readFrame();
    Q_INVOKABLE QString rawRead();
    Q_INVOKABLE bool rawWrite(quint32 id,const QByteArray& data);

    Q_INVOKABLE quint32 framesAvailable();
    void framesReceivedLinker();
    //void link();
    QCanBusDevice* _canBusDevice;
    QCanBusFrame _frame;
    quint32 _frameId;
    QByteArray _frameData;
private:

    QList<QCanBusDevice::Filter> _filterList;
    quint32 _deviceId;
    bool _isConnected;
    BitRate _bitRate;
    bool _loopBackEnable;




signals:
    void deviceIdChanged();
    void connected();
    void disconnected();
    void bitRateChanged();
    void framesReceived();
public slots:
};

#endif // CANBUS_H
