import QtQuick 2.9
import QtQuick.Controls 2.2
import QtGraphicalEffects 1.0
import io.qt.canbus 1.0
import io.qt.backend 1.0

ApplicationWindow {
    visible: true
    width: 1024
    height: 600
    title: qsTr("ProyectoPOO")
    background: Rectangle {
        RadialGradient {
               anchors.fill: parent
               angle: 90
               gradient: Gradient {
                   GradientStop { position: 0.0; color: "#555555" }
                   GradientStop { position: 1.0; color: "#212121" }
                   GradientStop { position: 1.0; color: (canBus.temp>450)?"#551100":
                                                        (canBus.temp>350)?"#441100":
                                                        (canBus.temp>250)?"#331100":
                                                        (canBus.temp>150)?"#221111":"#000000" }
               }
           }
       }


    SwipeView {
        id: swipeView
        anchors.fill: parent
        currentIndex: 0

        BackEnd{
            id: canBus

            onConnected: {
               console.log("Connected")
            }
            onDisconnected: {
                console.log("Disconnected")
            }
            onVoltageChanged: {
                statusIcons.iconColor3=(canBus.voltage>1.5)?"#00c853":statusIcons.iconColor2
                statusIcons.borderColor3=(canBus.voltage>1.5)?"#00c853":statusIcons.borderColor2
            }
            onVoltageWrong: {
                statusIcons.iconColor3="#FF0000"
                statusIcons.borderColor3="#FF0000"
            }

            onVelocityChanged: {
                speedometer.value = canBus.velocity
                statusIcons.iconColor2=(canBus.velocity<70)?"#00c853":statusIcons.iconColor2
                statusIcons.borderColor2=(canBus.velocity<70)?"#00c853":statusIcons.borderColor2
            }
            onVelocityMax:{
                statusIcons.iconColor2="#FF0000"
                statusIcons.borderColor2="#FF0000"
            }


            onBatteryChanged: {
                batery.value = canBus.battery
                statusIcons.iconColor1=(canBus.battery>10)?"#00c853":statusIcons.iconColor1
                statusIcons.borderColor1=(canBus.battery>10)?"#00c853":statusIcons.borderColor1
            }
            onCriticalBattery: {
                statusIcons.iconColor1="#FF0000"
                statusIcons.borderColor1="#FF0000"
            }

            onRpmChanged:{
                speedometer2.value = canBus.rpm
                statusIcons.iconColor5=(canBus.rpm<3)?"#00c853":statusIcons.iconColor5
                statusIcons.borderColor5=(canBus.rpm<3)?"#00c853":statusIcons.borderColor5
            }
            onRpmCritical:{
                statusIcons.iconColor5="#FF0000"
                statusIcons.borderColor5="#FF0000"
            }
            onCurrentChanged: {
                statusIcons.iconColor6=(canBus.current<5)?"#00c853":statusIcons.iconColor6
                statusIcons.borderColor6=(canBus.current<5)?"#00c853":statusIcons.borderColor6
            }
            onCurrentExceed: {
                statusIcons.iconColor6="#FF0000"
                statusIcons.borderColor6="#FF0000"
            }
            onHeightChanged: {
                statusIcons.iconColor9=(canBus.current<1000)?"#00c853":statusIcons.iconColor6
                statusIcons.borderColor9=(canBus.current<1000)?"#00c853":statusIcons.borderColor6
            }

            onHeightExceed: {
                statusIcons.iconColor9="#FF0000"
                statusIcons.borderColor9="#FF0000"
            }
            onTempChanged: {
                statusIcons.iconColor4=(canBus.current<450)?"#00c853":statusIcons.iconColor4
                statusIcons.borderColor4=(canBus.current<450)?"#00c853":statusIcons.borderColor4
            }
            onTempExceed: {
                statusIcons.iconColor4="#FF0000"
                statusIcons.borderColor4="#FF0000"
            }
            onHumidityChanged: {
                statusIcons.iconColor8=(canBus.current<450)?"#00c853":statusIcons.iconColor8
                statusIcons.borderColor8=(canBus.current<450)?"#00c853":statusIcons.borderColor8
            }
            onHumidityExceed: {
                statusIcons.iconColor8="#FF0000"
                statusIcons.borderColor8="#FF0000"
            }
            onPressureChanged: {
                statusIcons.iconColor7=(canBus.current<1500)?"#00c853":statusIcons.iconColor7
                statusIcons.borderColor7=(canBus.current<1500)?"#00c853":statusIcons.borderColor7
            }
            onPressureExceed: {
                statusIcons.iconColor7="#FF0000"
                statusIcons.borderColor7="#FF0000"
            }

        }

        Item {
            id:general

            Time {
                id: currentTime
                x: 1024/2 - currentTime.width/2 - 25
                y: 90
                fontSize: 12
                formatTime: "hh:mm:ss"
                textFontName: fontRoboto.name
            }

            Speedometer {
                id: speedometer
                x: 100
                y: 70
                gradient: Gradient{
                    GradientStop { position: 0.0; color: (speedometer.value>0.7*speedometer.maximumValue)?"#775555":
                                                         (speedometer.value>0.5*speedometer.maximumValue)?"#666655":
                                                                                                         "#556655" }
                    GradientStop { position: 1.0; color: "#212133" }
                    GradientStop { position: 2.0; color: "#000000" }
                }

                borderSize: 10
                backgroundColor: "#212121"
                foregroundColor: "#66bb6a"
                foregroundColorLight: "#00c866"
                fontName: fontRoboto.name

            }

            Speedometer {
                id: speedometer2
                x: 1024 - speedometer.x - speedometer2.width
                y: 70
                gradient: Gradient{
                    GradientStop { position: 0.0; color: (speedometer2.value>speedometer2.maximumValue/3)?"#775555":
                                                         (speedometer2.value>speedometer2.maximumValue/6)?"#666655":
                                                                                                         "#555566" }
                    GradientStop { position: 1.0; color: "#212121" }
                    GradientStop { position: 2.0; color: "#000000" }
                }
                borderSize: 10
                maximumValue: 9
                tickmarkStepSize1: 1        // Arreglar esto
                backgroundColor: "#212121"
                foregroundColor: "#66bb6a"
                foregroundColorLight: "#00A877"
                gaugeText: "x1000rpm"
                fontName: fontRoboto.name
            }

            Batery {
                id: batery
                x: 1024/2 - batery.width/2
                y: 270
                borderSize: 5
                width: 150
                height: 150
                minAngleValue: -180
                maxAngleValue: 180
                minimumValue: 0
                maximumValue: 100
                backgroundLineWidth: 0.1
                foregroundLineWidth: 0.1
                backgroundColor: "#212121"
                foregroundColor: "#212121"
                foregroundColorLight: "#00c853"
                fontName: fontRoboto.name

            }

            Switch{
                id: switchCan
                property bool butt_state: false
                checked: butt_state
                x: 8
                y: 50

                onClicked:{
                    if(switchCan.butt_state==false){
                        switchCan.butt_state = true
                        switchCan_details.butt_state=true
                        canBus.loopbackEnable()
                        canBus.connect()
                        console.log("on")
                    } else{
                        console.log("off")
                        switchCan.butt_state= false
                        switchCan_details.butt_state=false
                        canBus.disconnect()
                    }

                }
            }

            StatusIcons {
                id: statusIcons
                x: 1024/2 - statusIcons.width/2
                y: 450
                iconText1: ""
                iconText2: ""
                iconText3: "\uf0e7"
                iconText4: ""
                iconText5: ""
                iconText6: ""
                iconText7: "\uf071"
                iconText8: ""
                iconText9: ""
            }
        }

        Item {
            id:details

            Switch{
                id: switchCan_details
                property bool butt_state: false
                checked: butt_state
                x: 8
                y: 50

                onClicked:{
                    if(switchCan.butt_state==false){
                        switchCan.butt_state = true
                        switchCan_details.butt_state=true
                        canBus.loopbackEnable()
                        canBus.connect()
                        console.log("on")
                    } else{
                        console.log("off")
                        switchCan.butt_state= false
                        switchCan_details.butt_state=false
                        canBus.disconnect()
                    }

                }
            }
            DataShow {
                id:voltage_show
                x:100
                y:100

                nameText: "Voltaje"
                dataText: canBus.voltage.toFixed(2)
                dataColor: statusIcons.iconColor3
                borderColor: "#EEEEEE"
                boxColor: "#454545"
            }

            DataShow {
                id:current_show
                x:100+voltage_show.nameWidth-current_show.nameWidth
                y:180

                nameText: "Corriente"
                dataText: canBus.current.toFixed(2)
                dataColor: statusIcons.iconColor6
                borderColor: "#EEEEEE"
                boxColor: "#454545"
            }

            DataShow{
                id: rpm_show
                x:100+voltage_show.nameWidth-rpm_show.nameWidth
                y:260

                nameText: "RPM"
                dataText: canBus.rpm.toFixed(2)
                dataColor: statusIcons.iconColor5
                borderColor: "#EEEEEE"
                boxColor: "#454545"
            }
            DataShow{
                id: velocity_show
                x:100+voltage_show.nameWidth-velocity_show.nameWidth
                y:340

                nameText: "Velocidad"
                dataText: canBus.velocity.toFixed(2)
                dataColor: statusIcons.iconColor2
                borderColor: "#EEEEEE"
                boxColor: "#454545"
            }

            DataShow{
                id:battery_show
                x:100+voltage_show.nameWidth-battery_show.nameWidth
                y:420

                nameText: "Bateria"
                dataColor: statusIcons.iconColor1
                dataText: canBus.battery.toFixed(2)
                borderColor: "#EEEEEE"
                boxColor: "#454545"
            }

            DataShow{
                id: pressure_show
                x:350
                y:100

                nameText: "Presion"
                dataText: canBus.pressure.toFixed(1)
                dataColor: statusIcons.iconColor7
                borderColor: "#EEEEEE"
                boxColor: "#454545"
            }

            DataShow{
                id:humidity_show
                x:350+pressure_show.nameWidth-humidity_show.nameWidth
                y:180

                nameText: "Humedad"
                dataText: canBus.humidity.toFixed(2)
                dataColor: statusIcons.iconColor8
                borderColor: "#EEEEEE"
                boxColor: "#454545"
            }

            DataShow{
                id:height_show
                x:350+pressure_show.nameWidth-height_show.nameWidth
                y:260

                nameText: "Altura"
                dataText: canBus.height.toFixed(2)
                dataColor: statusIcons.iconColor9
                borderColor: "#EEEEEE"
                boxColor: "#454545"

            }
            DataShow{
                id:temp_show
                x:350+pressure_show.nameWidth-temp_show.nameWidth
                y:340

                nameText: "Temperatura"
                dataText: canBus.temp.toFixed(2)
                dataColor: statusIcons.iconColor4
                borderColor: "#EEEEEE"
                boxColor: "#454545"
            }

            Button {
                id: bitRate
                text: "BitRate :"+canBus.bitRate
                x:600
                y:100
                onClicked: menu.open()
                /*
        Bitrate50000 = 50000,
        Bitrate100000 = 100000,
        Bitrate250000 = 250000,
        Bitrate500000 = 500000,
        Bitrate800000 = 800000,
        Bitrate1000000 = 1000000*/
                Menu {
                    id: menu

                    MenuItem {
                        text: "50000"
                        onTriggered: {
                            canBus.setBitRate(50000)
                        }

                    }
                    MenuItem {
                        text: "100000"
                        onTriggered: {
                            canBus.setBitRate(100000)
                        }
                    }
                    MenuItem {
                        text: "250000"
                        onTriggered: {
                            canBus.setBitRate(250000)
                        }
                    }
                    MenuItem {
                        text: "500000"
                        onTriggered: {
                            canBus.setBitRate(500000)
                        }
                    }
                    MenuItem {
                        text: "800000"
                        onTriggered: {
                            canBus.setBitRate(800000)
                        }
                    }
                    MenuItem {
                        text: "1000000"
                        onTriggered: {
                            canBus.setBitRate(1000000)
                        }
                    }
                }
            }

        }

    }
    footer: TabBar {
        id: tabBar
        currentIndex: swipeView.currentIndex
        background: Rectangle {
                color:"#212121"
        }

        TabButton {
            id: generalTab
            onPressed: {
                swipeView.decrementCurrentIndex()
            }

            Rectangle {
                gradient: Gradient {
                    GradientStop { position: 0; color: "#555555" }
                    GradientStop { position: 1; color: "#353535" }
                    GradientStop { position: 2; color: "#212121" }
                }
                anchors.fill: parent
                //color: tabBar.currentIndex == 0 ? "#424242" : "#212121"
                color: "#212121"
                Text {
                    anchors.verticalCenter: parent.verticalCenter
                    anchors.horizontalCenter: parent.horizontalCenter
                    //color: "#F5F5F5"
                    color: tabBar.currentIndex == 0 ? "#00c853" : "#F5F5F5"
                    text: qsTr("General")
                }
            }
        }
        TabButton {
            id:detailsTab
            onPressed: {
                swipeView.incrementCurrentIndex()
            }

            Rectangle {
                gradient: Gradient {
                    GradientStop { position: 0; color: "#555555" }
                    GradientStop { position: 1; color: "#353535" }
                    GradientStop { position: 2; color: "#212121" }
                }
                anchors.fill: parent
                //color: tabBar.currentIndex == 1 ? "#424242" : "#212121"
                color: "#212121"
                Text {
                    anchors.verticalCenter: parent.verticalCenter
                    anchors.horizontalCenter: parent.horizontalCenter
                    //color: "#F5F5F5"
                    color: tabBar.currentIndex == 1 ? "#00c853" : "#F5F5F5"
                    text: qsTr("Details")
                }
            }
        }
    }

    /* Font */
    FontLoader {
        id: fa
        source: "./fonts/fontawesome-webfont.ttf"
    }

    FontLoader {
        id: faBrands
        source: "./fonts/fa-brands-400.ttf"
    }

    FontLoader {
        id: faRegular
        source: "./fonts/fa-regular-400.ttf"
    }

    FontLoader {
        id: faSolid
        source: "./fonts/fa-solid-900.ttf"
    }

    FontLoader {
        id: fontRoboto
        source: "./fonts/Roboto-Regular.ttf"
    }
}

